package rym_util

import (
	"bytes"
	"fmt"
	"gorm.io/gorm/logger"
	"runtime"
	"strconv"
	"strings"
	"time"
)

const (
	DebugL = "Debug"
	InfoL  = "Info"
	ErrorL = "Error"
)

func DebugDo(f func()) {
	if IsDebug() {
		f()
	}
}

func IsDebug() bool {
	ok := false
	switch strings.ToLower(LogLevel) {
	case strings.ToLower(DebugL):
		ok = true
		break
	}
	return ok
}

func Debug(msg string) {
	ok := false
	switch strings.ToLower(LogLevel) {
	case strings.ToLower(DebugL):
		ok = true
		break
	}
	if ok {
		_, file, line, _ := runtime.Caller(1)
		mark("Debug", msg, file, line)
	}
}

func Info(msg string) {
	ok := false
	switch strings.ToLower(LogLevel) {
	case strings.ToLower(DebugL):
		ok = true
		break
	case strings.ToLower(InfoL):
		ok = true
		break
	}
	if ok {
		_, file, line, _ := runtime.Caller(1)
		mark("Info", msg, file, line)
	}
}

func Error(msg string) {
	ok := false
	switch strings.ToLower(LogLevel) {
	case strings.ToLower(DebugL):
		ok = true
		break
	case strings.ToLower(InfoL):
		ok = true
		break
	case strings.ToLower(ErrorL):
		ok = true
		break
	}
	if ok {
		_, file, line, _ := runtime.Caller(1)
		mark("Error", msg, file, line)
	}
}

func mark(tag, msg string, file string, line int) {
	str := "%s %s [%d] %s %d %s"
	if LogColorful {
		if tag == "Error" {
			str = "%s %s [%d] %s %d " + logger.Red + "%s" + logger.Reset
		} else if tag == "Info" {
			str = "%s %s [%d] %s %d " + logger.Yellow + "%s" + logger.Reset
		} else {
			str = "%s %s [%d] %s %d " + logger.Green + "%s" + logger.Reset
		}
	}
	file = file[strings.LastIndex(file, "/")+1:]
	fmt.Println(fmt.Sprintf(str, FormatDateMillTime(time.Now()), tag, GetGID(), file, line, msg))
}

func FormatDateMillTime(dataTime time.Time) string {
	return dataTime.Format("2006-01-02 15:04:05.000000")
}

func GetGID() uint64 {
	b := make([]byte, 64)
	b = b[:runtime.Stack(b, false)]
	b = bytes.TrimPrefix(b, []byte("goroutine "))
	b = b[:bytes.IndexByte(b, ' ')]
	n, _ := strconv.ParseUint(string(b), 10, 64)
	return n
}
